@@ -133,10 +133,6 @@ local function path_is_multi_type(path, type_val)
         return true
     end
 
-    if path == "apisix->ssl->listen_port" and type_val == "number" then
-        return true
-    end
-
     return false
 end
 
@@ -237,21 +233,8 @@ function _M.read_yaml_conf(apisix_home)
         end
     end
 
-    if default_conf.apisix.config_center == "yaml" then
-        local apisix_conf_path = profile:yaml_path("apisix")
-        local apisix_conf_yaml, _ = util.read_file(apisix_conf_path)
-        if apisix_conf_yaml then
-            local apisix_conf = yaml.parse(apisix_conf_yaml)
-            if apisix_conf then
-                local ok, err = resolve_conf_var(apisix_conf)
-                if not ok then
-                    return nil, err
-                end
-            end
-        end
-    end
-
     if default_conf.deployment then
+        default_conf.deployment.config_provider = "etcd"
         if default_conf.deployment.role == "traditional" then
             default_conf.etcd = default_conf.deployment.etcd
 
@@ -261,7 +244,9 @@ function _M.read_yaml_conf(apisix_home)
 
         elseif default_conf.deployment.role == "data_plane" then
             if default_conf.deployment.role_data_plane.config_provider == "yaml" then
-                default_conf.apisix.config_center = "yaml"
+                default_conf.deployment.config_provider = "yaml"
+            elseif default_conf.deployment.role_data_plane.config_provider == "xds" then
+                default_conf.deployment.config_provider = "xds"
             else
                 default_conf.etcd = default_conf.deployment.role_data_plane.control_plane
             end
@@ -280,6 +265,20 @@ function _M.read_yaml_conf(apisix_home)
         end
     end
 
+    if default_conf.deployment.config_provider == "yaml" then
+        local apisix_conf_path = profile:yaml_path("apisix")
+        local apisix_conf_yaml, _ = util.read_file(apisix_conf_path)
+        if apisix_conf_yaml then
+            local apisix_conf = yaml.parse(apisix_conf_yaml)
+            if apisix_conf then
+                local ok, err = resolve_conf_var(apisix_conf)
+                if not ok then
+                    return nil, err
+                end
+            end
+        end
+    end
+
     return default_conf
 end
 
