@@ -21,6 +21,7 @@
 
 local table        = require("apisix.core.table")
 local config_local = require("apisix.core.config_local")
+local config_util  = require("apisix.core.config_util")
 local log          = require("apisix.core.log")
 local json         = require("apisix.core.json")
 local etcd_apisix  = require("apisix.core.etcd")
@@ -212,14 +213,17 @@ local function load_full_data(self, dir_res, headers)
         self:upgrade_version(item.modifiedIndex)
 
     else
-        if not dir_res.nodes then
-            dir_res.nodes = {}
+        -- here dir_res maybe res.body.node or res.body.list
+        -- we need make values equals to res.body.node.nodes or res.body.list
+        local values = (dir_res and dir_res.nodes) or dir_res
+        if not values then
+            values = {}
         end
 
-        self.values = new_tab(#dir_res.nodes, 0)
-        self.values_hash = new_tab(0, #dir_res.nodes)
+        self.values = new_tab(#values, 0)
+        self.values_hash = new_tab(0, #values)
 
-        for _, item in ipairs(dir_res.nodes) do
+        for _, item in ipairs(values) do
             local key = short_key(self, item.key)
             local data_valid = true
             if type(item.value) ~= "table" then
@@ -302,7 +306,7 @@ local function sync_data(self)
             return false, err
         end
 
-        local dir_res, headers = res.body.node or {}, res.headers
+        local dir_res, headers = res.body.list or {}, res.headers
         log.debug("readdir key: ", self.key, " res: ",
                   json.delay_encode(dir_res))
         if not dir_res then
@@ -311,12 +315,7 @@ local function sync_data(self)
 
         if self.values then
             for i, val in ipairs(self.values) do
-                if val and val.clean_handlers then
-                    for _, clean_handler in ipairs(val.clean_handlers) do
-                        clean_handler(val)
-                    end
-                    val.clean_handlers = nil
-                end
+                config_util.fire_all_clean_handlers(val)
             end
 
             self.values = nil
@@ -403,11 +402,8 @@ local function sync_data(self)
         local pre_index = self.values_hash[key]
         if pre_index then
             local pre_val = self.values[pre_index]
-            if pre_val and pre_val.clean_handlers then
-                for _, clean_handler in ipairs(pre_val.clean_handlers) do
-                    clean_handler(pre_val)
-                end
-                pre_val.clean_handlers = nil
+            if pre_val then
+                config_util.fire_all_clean_handlers(pre_val)
             end
 
             if res.value then
@@ -511,7 +507,7 @@ do
         end
 
         local err
-        etcd_cli, err = etcd_apisix.new()
+        etcd_cli, err = etcd_apisix.switch_proxy()
         return etcd_cli, err
     end
 end
