@@ -109,6 +109,10 @@ function _M.http_init_worker()
     -- for testing only
     core.log.info("random test in [1, 10000]: ", math.random(1, 10000))
 
+    -- Because go's scheduler doesn't work after fork, we have to load the gRPC module
+    -- in each worker.
+    core.grpc = require("apisix.core.grpc")
+
     local we = require("resty.worker.events")
     local ok, err = we.configure({shm = "worker-events", interval = 0.1})
     if not ok then
@@ -152,6 +156,9 @@ end
 
 
 function _M.http_exit_worker()
+    -- TODO: we can support stream plugin later - currently there is not `destory` method
+    -- in stream plugins
+    plugin.exit_worker()
     require("apisix.plugins.ext-plugin.init").exit_worker()
 end
 
@@ -229,11 +236,6 @@ local function set_upstream_headers(api_ctx, picked_server)
     if proto then
         api_ctx.var.var_x_forwarded_proto = proto
     end
-
-    local x_forwarded_host = api_ctx.var.http_x_forwarded_host
-    if x_forwarded_host then
-        api_ctx.var.var_x_forwarded_host = x_forwarded_host
-    end
 end
 
 
@@ -267,51 +269,6 @@ local function verify_tls_client(ctx)
 end
 
 
-local function verify_https_client(ctx)
-    local scheme = ctx.var.scheme
-    if scheme ~= "https" then
-        return true
-    end
-
-    local host = ctx.var.host
-    local matched = router.router_ssl.match_and_set(ctx, true, host)
-    if not matched then
-        return true
-    end
-
-    local matched_ssl = ctx.matched_ssl
-    if matched_ssl.value.client and apisix_ssl.support_client_verification() then
-        local verified = apisix_base_flags.client_cert_verified_in_handshake
-        if not verified then
-            -- vanilla OpenResty requires to check the verification result
-            local res = ctx.var.ssl_client_verify
-            if res ~= "SUCCESS" then
-                if res == "NONE" then
-                    core.log.error("client certificate was not present")
-                else
-                    core.log.error("client certificate verification is not passed: ", res)
-                end
-
-                return false
-            end
-        end
-
-        local sni = apisix_ssl.server_name()
-        if sni ~= host then
-            -- There is a case that the user configures a SSL object with `*.domain`,
-            -- and the client accesses with SNI `a.domain` but uses Host `b.domain`.
-            -- This case is complex and we choose to restrict the access until there
-            -- is a stronge demand in real world.
-            core.log.error("client certificate verified with SNI ", sni,
-                           ", but the host is ", host)
-            return false
-        end
-    end
-
-    return true
-end
-
-
 local function normalize_uri_like_servlet(uri)
     local found = core.string.find(uri, ';')
     if not found then
@@ -374,12 +331,12 @@ function _M.http_access_phase()
     local api_ctx = core.tablepool.fetch("api_ctx", 0, 32)
     ngx_ctx.api_ctx = api_ctx
 
-    core.ctx.set_vars_meta(api_ctx)
-
-    if not verify_https_client(api_ctx) then
+    if not verify_tls_client(api_ctx) then
         return core.response.exit(400)
     end
 
+    core.ctx.set_vars_meta(api_ctx)
+
     debug.dynamic_debug(api_ctx)
 
     local uri = api_ctx.var.uri
@@ -574,10 +531,6 @@ function _M.http_access_phase()
         core.log.info("enabled websocket for route: ", route.value.id)
     end
 
-    if route.value.service_protocol == "grpc" then
-        api_ctx.upstream_scheme = "grpc"
-    end
-
     -- load balancer is not required by kafka upstream, so the upstream
     -- node selection process is intercepted and left to kafka to
     -- handle on its own
@@ -813,7 +766,7 @@ end
 
 local function cors_admin()
     local_conf = core.config.local_conf()
-    if local_conf.apisix and not local_conf.apisix.enable_admin_cors then
+    if not core.table.try_read_attr(local_conf, "deployment", "admin", "enable_admin_cors") then
         return
     end
 
