@@ -81,10 +81,6 @@ local function check_disable(plugin_conf)
         return nil
     end
 
-    if plugin_conf.disable then
-        return plugin_conf.disable
-    end
-
     if not plugin_conf._meta then
        return nil
     end
@@ -162,14 +158,6 @@ local function load_plugin(name, plugins_list, plugin_type)
     local plugin_injected_schema = core.schema.plugin_injected_schema
 
     if plugin.schema['$comment'] ~= plugin_injected_schema['$comment'] then
-        if properties.disable then
-            core.log.error("invalid plugin [", name,
-                           "]: found forbidden 'disable' field in the schema")
-            return
-        end
-
-        properties.disable = plugin_injected_schema.disable
-
         if properties._meta then
             core.log.error("invalid plugin [", name,
                            "]: found forbidden '_meta' field in the schema")
@@ -180,7 +168,6 @@ local function load_plugin(name, plugins_list, plugin_type)
         -- new injected fields should be added under `_meta`
         -- 1. so we won't break user's code when adding any new injected fields
         -- 2. the semantics is clear, especially in the doc and in the caller side
-        -- TODO: move the `disable` to `_meta` too
 
         plugin.schema['$comment'] = plugin_injected_schema['$comment']
     end
@@ -369,6 +356,24 @@ function _M.load(config)
 end
 
 
+function _M.exit_worker()
+    for name, plugin in pairs(local_plugins_hash) do
+        local ty = PLUGIN_TYPE_HTTP
+        if plugin.type == "wasm" then
+            ty = PLUGIN_TYPE_HTTP_WASM
+        end
+        unload_plugin(name, ty)
+    end
+
+    -- we need to load stream plugin so that we can check their schemas in
+    -- Admin API. Maybe we can avoid calling `load` in this case? So that
+    -- we don't need to call `destroy` too
+    for name in pairs(stream_local_plugins_hash) do
+        unload_plugin(name, PLUGIN_TYPE_STREAM)
+    end
+end
+
+
 local function trace_plugins_info_for_debug(ctx, plugins)
     if not enable_debug() then
         return
@@ -410,13 +415,6 @@ local function meta_filter(ctx, plugin_name, plugin_conf)
         return true
     end
 
-    local match_cache_key =
-        ctx.conf_type .. "#" .. ctx.conf_id .. "#"
-            .. ctx.conf_version .. "#" .. plugin_name .. "#meta_filter_matched"
-    if ctx[match_cache_key] ~= nil then
-        return ctx[match_cache_key]
-    end
-
     local ex, ok, err
     if ctx then
         ex, err = expr_lrucache(plugin_name .. ctx.conf_type .. ctx.conf_id,
@@ -429,14 +427,12 @@ local function meta_filter(ctx, plugin_name, plugin_conf)
                          " plugin_name: ", plugin_name)
         return true
     end
-    ok, err = ex:eval(ctx.var)
+    ok, err = ex:eval()
     if err then
         core.log.warn("failed to run the 'vars' expression: ", err,
                          " plugin_name: ", plugin_name)
         return true
     end
-
-    ctx[match_cache_key] = ok
     return ok
 end
 
@@ -461,7 +457,9 @@ function _M.filter(ctx, conf, plugins, route_conf, phase)
             goto continue
         end
 
-        if not check_disable(plugin_conf) then
+        local matched = meta_filter(ctx, name, plugin_conf)
+        local disable = check_disable(plugin_conf)
+        if not disable and matched then
             if plugin_obj.run_policy == "prefer_route" and route_plugin_conf ~= nil then
                 local plugin_conf_in_route = route_plugin_conf[name]
                 local disable_in_route = check_disable(plugin_conf_in_route)
@@ -543,7 +541,8 @@ function _M.stream_filter(user_route, plugins)
         local name = plugin_obj.name
         local plugin_conf = user_plugin_conf[name]
 
-        if type(plugin_conf) == "table" and not plugin_conf.disable then
+        local disable = check_disable(plugin_conf)
+        if type(plugin_conf) == "table" and not disable then
             core.table.insert(plugins, plugin_obj)
             core.table.insert(plugins, plugin_conf)
         end
@@ -658,7 +657,7 @@ function _M.merge_consumer_route(route_conf, consumer_conf, api_ctx)
 
     local flag = route_conf.value.id .. "#" .. route_conf.modifiedIndex
                  .. "#" .. consumer_conf.id .. "#" .. consumer_conf.modifiedIndex
-    local new_conf = merged_route(flag, api_ctx.conf_version,
+    local new_conf = merged_route(flag, nil,
                         merge_consumer_route, route_conf, consumer_conf)
 
     api_ctx.conf_type = api_ctx.conf_type .. "&consumer"
@@ -790,9 +789,6 @@ local function check_single_plugin_schema(name, plugin_conf, schema_type, skip_d
     end
 
     if plugin_obj.check_schema then
-        local disable = plugin_conf.disable
-        plugin_conf.disable = nil
-
         local ok, err = plugin_obj.check_schema(plugin_conf, schema_type)
         if not ok then
             return false, "failed to check the configuration of plugin "
@@ -805,8 +801,6 @@ local function check_single_plugin_schema(name, plugin_conf, schema_type, skip_d
                 return nil, "failed to validate the 'vars' expression: " .. err
             end
         end
-
-        plugin_conf.disable = disable
     end
 
     return true
@@ -854,16 +848,11 @@ local function stream_check_schema(plugins_conf, schema_type, skip_disabled_plug
         end
 
         if plugin_obj.check_schema then
-            local disable = plugin_conf.disable
-            plugin_conf.disable = nil
-
             local ok, err = plugin_obj.check_schema(plugin_conf, schema_type)
             if not ok then
                 return false, "failed to check the configuration of "
                               .. "stream plugin [" .. name .. "]: " .. err
             end
-
-            plugin_conf.disable = disable
         end
 
         ::CONTINUE::
@@ -925,12 +914,8 @@ function _M.run_plugin(phase, plugins, api_ctx)
             end
 
             if phase_func then
-                local conf = plugins[i + 1]
-                if not meta_filter(api_ctx, plugins[i]["name"], conf)then
-                    goto CONTINUE
-                end
-
                 plugin_run = true
+                local conf = plugins[i + 1]
                 local code, body = phase_func(conf, api_ctx)
                 if code or body then
                     if is_http then
@@ -963,10 +948,9 @@ function _M.run_plugin(phase, plugins, api_ctx)
 
     for i = 1, #plugins, 2 do
         local phase_func = plugins[i][phase]
-        local conf = plugins[i + 1]
-        if phase_func and meta_filter(api_ctx, plugins[i]["name"], conf) then
+        if phase_func then
             plugin_run = true
-            phase_func(conf, api_ctx)
+            phase_func(plugins[i + 1], api_ctx)
         end
     end
 
