@@ -159,7 +159,7 @@ end
 
 function _M.init()
     local err
-    protos, err = core.config.new("/proto", {
+    protos, err = core.config.new("/protos", {
         automatic = true,
         item_schema = core.schema.proto
     })
@@ -170,10 +170,5 @@ function _M.init()
     end
 end
 
-function _M.destroy()
-    if protos then
-        protos:close()
-    end
-end
 
 return _M
