@@ -143,11 +143,6 @@ function _M.log(conf, ctx)
         end
     end
 
-    local service_instance_name = conf.service_instance_name
-    if service_instance_name == "$hostname" then
-        service_instance_name = core.utils.gethostname()
-    end
-
     local entry = {
         traceContext = trace_context,
         body = {
@@ -156,7 +151,7 @@ function _M.log(conf, ctx)
             }
         },
         service = conf.service_name,
-        serviceInstance = service_instance_name,
+        serviceInstance = conf.service_instance_name,
         endpoint = ctx.var.uri,
     }
 
