@@ -172,11 +172,7 @@ local function set_upstream(upstream_info, ctx)
         upstream_host = upstream_info.upstream_host,
         key = upstream_info.key,
         nodes = new_nodes,
-        timeout = {
-            send = upstream_info.timeout and upstream_info.timeout.send or 15,
-            read = upstream_info.timeout and upstream_info.timeout.read or 15,
-            connect = upstream_info.timeout and upstream_info.timeout.connect or 15
-        }
+        timeout = upstream_info.timeout,
     }
 
     local ok, err = upstream.check_schema(up_conf)
