@@ -596,9 +596,6 @@ _M.route = {
 
         service_id = id_schema,
         upstream_id = id_schema,
-        service_protocol = {
-            enum = {"grpc", "http"}
-        },
 
         enable_websocket = {
             description = "enable websocket for request",
@@ -926,12 +923,12 @@ _M.id_schema = id_schema
 
 _M.plugin_injected_schema = {
     ["$comment"] = "this is a mark for our injected plugin schema",
-    disable = {
-        type = "boolean",
-    },
     _meta = {
         type = "object",
         properties = {
+            disable = {
+                type = "boolean",
+            },
             error_response = {
                 oneOf = {
                     { type = "string" },
@@ -946,10 +943,6 @@ _M.plugin_injected_schema = {
                 description = "filter determines whether the plugin "..
                                 "needs to be executed at runtime",
                 type  = "array",
-                maxItems = 20,
-                items = {
-                    type = "array"
-                }
             }
         }
     }
